# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['asp_clean']

package_data = \
{'': ['*']}

install_requires = \
['openpyxl>=3.0.5,<4.0.0', 'requests>=2.24.0,<3.0.0', 'toolz>=0.10.0,<0.11.0']

setup_kwargs = {
    'name': 'asp-clean',
    'version': '0.1.1',
    'description': '',
    'long_description': None,
    'author': 'YewtsingMu',
    'author_email': 'mooyewtsing@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.6.9,<4.0.0',
}


setup(**setup_kwargs)
